/*
gillespie2.c

Wrapper executable for the exact discrete event simulation algorithm
contained in "simulate.c"

Last edited on: $LastChangedDate: 2006-01-25 13:10:57 +0000 (Wed, 25 Jan 2006) $
by $Author: colin $
Revision: $Rev: 54 $
*/
#include "simulate.h"

#include <getopt.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>

int main(int argc,char *argv[])
{
  SBMLDocument_t *doc;
  Model_t *mod;
  gsl_rng *r;
  int opt,fileref,numSteps,processid,count,numCommas;
  unsigned long seed,maxint=999999999;
  double termtime;
  char *filename;
  char *speciesList;
  char temp;
  char *temp2;
  char** explodedarray;

  /* default args */
  filename=NULL;
  speciesList=NULL;
  termtime=0.0;
  fileref=0;
  processid=getpid();
  seed=0XDEECE66D *((unsigned)time(NULL)+processid) + 0x2BBB62DC;
  if(seed>maxint) seed=(long)(seed/100.0); /*length of seed allowed in database*/
  numSteps=1000;
  
  numCommas=0;
  explodedarray='\0';

  /* process command line */
  while ( (opt=getopt(argc,argv,"m:t:s:r:n:p:h")) != -1 ) {
    switch(opt) {
    case 'm':
      filename=optarg; break;
    case 't':
      termtime=atof(optarg); break;
    case 's':
      seed=atoi(optarg); break;
    case 'r':
      fileref=atoi(optarg); break;
    case 'n':
       numSteps=atoi(optarg);break;  
    case 'p':
      speciesList=optarg; 
      /* Calculate the number of commas. */
      count = 0;
      numCommas=0;
      temp=(char)speciesList[count];
      while (temp != '\0') {
        if (temp == ',') { numCommas++; }
        count++;
	temp=(char)speciesList[count];
      }
      
      /* Explode the string to be all the substrings between commas. */
      explodedarray = (char**) malloc(sizeof(char*)*(numCommas+1));
      
      temp2 = (char*)strtok(speciesList,",");
      count = 0;
      while (temp2 != NULL) {
        explodedarray[count] = temp2;
        temp2 = (char*)strtok(NULL,",");
        count++;
      }
      /*
      count = 0;
      printf("\n");
      while (count < numCommas+1) {
        printf("Stored:  %s\n",explodedarray[count]);
	count++;
      }*/
      
      
      break;
    case 'h':
      fprintf(stderr,"\ngillespie - simple implementation of the gillespie algorithm\n\n");
      fprintf(stderr,"Usage: %s (-m <modFile> OR -r <fileRef>)  -t <termTime> -p <speciesToOutput> [ -s <seed> ] [-n <numSteps>] \nDefaults: -s random seed \n\t  -n numSteps %d\n",argv[0],numSteps);
      fprintf(stderr,"Arguments:\n\t<modFile>\tFilename of SBML model\n\t<fileRef>\tReference Number of SBML model\n\t<termTime>\tTime at which to end simulation\n\t<speciesToOutput>\tA column separated list of species to output\n\t<seed>\t\tInteger seed for GSL RNG\n\t<numSteps>\tNumber of timesteps output to file\n");
      fprintf(stderr,"\n");
      exit(EXIT_FAILURE);
    case ':':
      fprintf(stderr,"Option requiring a value given without a value\n");
      exit(EXIT_FAILURE);
    case '?':
      fprintf(stderr,"Unknown option: %c\n",optopt);
      exit(EXIT_FAILURE);
    }
  }
  
  /* check compulsory args */
  if ( ((fileref==0)&&(filename==NULL)) || (termtime==0.0) || (speciesList == NULL)) {
    fprintf(stderr,"Arguments to options (-m or -r), -p and -t are compulsory.\nDo '%s -h' for a usage summary.\n",argv[0]);
    exit(EXIT_FAILURE);
  }
  
  r=gsl_rng_alloc(gsl_rng_mt19937);
  if (seed>0) {
    gsl_rng_set(r,seed);
  }
  
  if(filename!=NULL) doc=readSBML(filename);
  else doc=dbGetSBML(fileref, seed); 
  SBMLDocument_printWarnings(doc,stderr);
  SBMLDocument_printErrors(doc,stderr);
  SBMLDocument_printFatals(doc,stderr);
  mod=SBMLDocument_getModel(doc);
  if(mod==NULL) fprintf(stderr, "Error: file %s doesn't exist!\n",filename);
  simulate(mod,stdout,r,termtime,fileref,numSteps,explodedarray,numCommas+1);
  dbFinish(fileref);
  return 0;
}

/* eof */

